/*
// Created by Tiny on 3/3/2021.
*/

/*!
 * \file GAME.c
 * \brief Game Source File
 * \authors Tin
 * \version 1
 * \date 03/03/2021
 *
 * Explanation.
 *
 */

#include <stdio.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include "GAME.h"
#include "prof.h"
#include "personnage.h"
#include "map.h"
#include "DS.h"

/*!
 * \fn SDL_Texture * query_hero(Personnage * hero, SDL_Rect * hero_rect, SDL_Renderer * renderer, SDL_Window * window)
 * \brief This function query a picture into a texture for hero
 *
 * \param [in] hero the hero pointer
 * \param [in] hero_rect the pointer to the structure rect of hero
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 */
SDL_Texture * query_hero(Personnage * hero, SDL_Rect * hero_rect, SDL_Renderer * renderer, SDL_Window * window){

    SDL_Texture * texture_hero;
    init_pers(hero,3,480,28,23,SPEED,"../ressource/man.png");

    // load the image data into the graphics hardware's memory
    texture_hero = IMG_LoadTexture(renderer, hero->image_pers);

    if (!texture_hero)
    {
        printf("Cannot create texture for persson: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return NULL;
    }

    // get and scale the dimensions of texture
    SDL_QueryTexture(texture_hero, NULL, NULL, &hero_rect->w, &hero_rect->h);
    hero_rect->w =  hero->larg_pers;
    hero_rect->h =  hero->long_pers;

    return texture_hero;
}

/*!
 * \fn SDL_Texture * query_teacher(Prof * teacher, SDL_Rect * teacher_rect, SDL_Renderer * renderer, SDL_Window * window )
 * \brief This function query a picture into a texture for teacher
 *
 * \param [in] teacher the teacher pointer
 * \param [in] teacher_rect the pointer to the structure rect of teacher
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 */
 
SDL_Texture * query_teacher(Prof * teacher, SDL_Rect * teacher_rect, SDL_Renderer * renderer, SDL_Window * window ){

    SDL_Texture * texture_teacher;
    init_prof(teacher,350,380,40,25,SPEED,"../ressource/prof.png");

    // load the image data into the graphics hardware's memory
    texture_teacher = IMG_LoadTexture(renderer, teacher->image_prof);

    if (!texture_teacher)
    {
        printf("Cannot create texture for prof: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return NULL;
    }

// get and scale the dimensions of texture
    SDL_QueryTexture(texture_teacher, NULL, NULL, &teacher_rect->w, &teacher_rect->h);
    teacher_rect->w = teacher->larg_prof;
    teacher_rect->h = teacher->long_prof;
    teacher_rect->x = teacher->x_prof;
    teacher_rect->y = teacher->y_prof;

    return texture_teacher;
}

/*!
 * \fn SDL_Texture * query_exam(DS * exam, SDL_Rect * exam_rect, SDL_Renderer * renderer, SDL_Window * window)
 * \brief This function query a picture into a texture for exam
 *
 * \param [in] exam the exam pointer
 * \param [in] exam_rect the pointer to the structure rect of exam
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 */
 
SDL_Texture * query_exam(DS * exam, SDL_Rect * exam_rect, SDL_Renderer * renderer, SDL_Window * window){

    fflush(stdin);
    int r = 2 + rand() % 29;

    SDL_Texture * texture_exam;
    init_DS(exam,r,"../ressource/DS.png");

    // load the image data into the graphics hardware's memory
    texture_exam = IMG_LoadTexture(renderer, exam->image_DS);

    if (!texture_exam)
    {
        printf("error creating texture: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return NULL;
    }

    SDL_QueryTexture(texture_exam, NULL, NULL, &exam_rect->w, &exam_rect->h);
    exam_rect->w /= 30;
    exam_rect->h /= 35;
    exam_rect->x=exam->x_DS;
    exam_rect->y=exam->y_DS;

    return texture_exam;
}


/*!
 * \fn SDL_Texture * query_map(MAP * map_game, SDL_Renderer * renderer, SDL_Window * window)
 * \brief This function query a picture into a texture for map
 *
 * \param [in] map_game the map of game's pointer
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 */
SDL_Texture * query_map(MAP * map_game, SDL_Renderer * renderer, SDL_Window * window){
    init_map(map_game,"../ressource/colored.bmp","../ressource/mapvis.bmp");
    map_game->functional = maptomatrix(map_game);

    //Create a texture for map
    SDL_Texture * texture_map = IMG_LoadTexture(renderer, map_game->visual_path);

    if (!texture_map)
    {
        printf("error creating texture for map: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return NULL;
    }

    return texture_map;
}

/*!
 * \fn int show_image_in_time(SDL_Renderer* renderer, SDL_Window * window, char * path, int time)
 * \brief This function show an image in some duration
 *
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 * \param [in] path the path of image
 * \param [in] time the duration
 */

int show_image_in_time(SDL_Renderer* renderer, SDL_Window * window, char * path, int time) {
//texture for beginning page
    SDL_Texture *texture_begin_page;
    texture_begin_page = IMG_LoadTexture(renderer, path);
    if (!texture_begin_page) {
        printf("error creating texture: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }
    SDL_RenderCopy(renderer, texture_begin_page, NULL, NULL);
    SDL_RenderPresent(renderer);
    SDL_Delay(time);
    SDL_DestroyTexture(texture_begin_page);
    return 0;
}
/*!
 * \fn void result_game(int result, SDL_Renderer * renderer, SDL_Window * window)
 * \brief This function test the result of the game
 *
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 * \param [in] result the result of game
 */

void result_game(int result, SDL_Renderer * renderer, SDL_Window * window){
    if(result==1){
        show_image_in_time(renderer,window,"../ressource/win.png",2000);
    }
    else show_image_in_time(renderer,window,"../ressource/lose.png",2000);
}


/*!
 * \fn void test_event_hero(Personnage * hero, SDL_Event * event, int * close)
 * \brief This function test the events of keyboard and change the model of hero and close_request if necessary
 *
 * \param [in] renderer the renderer pointer
 * \param [in] window the window pointer
 * \param [in] result the result of game
 */

void test_event_hero(Personnage * hero, SDL_Event * event, int * close){
    while (SDL_PollEvent(event))
    {
        switch (event->type)
        {
            case SDL_QUIT:
                *close = 1;
                break;
            case SDL_KEYDOWN:
                switch (event->key.keysym.scancode)
                {
                    case SDL_SCANCODE_UP:
                        hero->direction_personnage.up = 1;
                        break;
                    case SDL_SCANCODE_LEFT:
                        hero->direction_personnage.left = 1;
                        break;
                    case SDL_SCANCODE_DOWN:
                        hero->direction_personnage.down = 1;
                        break;
                    case SDL_SCANCODE_RIGHT:
                        hero->direction_personnage.right = 1;
                        break;
                }
                break;
            case SDL_KEYUP:
                switch (event->key.keysym.scancode)
                {
                    case SDL_SCANCODE_UP:
                        hero->direction_personnage.up = 0;
                        break;
                    case SDL_SCANCODE_LEFT:
                        hero->direction_personnage.left = 0;
                        break;
                    case SDL_SCANCODE_DOWN:
                        hero->direction_personnage.down = 0;
                        break;
                    case SDL_SCANCODE_RIGHT:
                        hero->direction_personnage.right = 0;
                        break;
                }
                break;
        }
    }
}
